#! /bin/bash

#  smindev.sh 
#
#    a script for studying the effects of varying s and mindev

# range of values for testing

svals="1 3.2e-1 1e-1 3.2e-2 1e-2"
mvals=`seq 0.025 0.025 0.47501`		# wide value range 
#mvals=`seq 0.420 0.020 0.46001` 	# high value range

# approx 0.1%-0.3% of nonspam corpus size for selecting spam_cutoff value

target=6	

# path for bogofilter executables

path="."

# file names

DATE=`date "+%m%d.%H%M"`

CFG="smindev.cf"
PARM_TBL="parms.$DATE.tbl"
RESULTS="results.$DATE.txt"

########## Code Begins Here ##########

# validate program paths

error="n"
for prog in bogofilter bogolexer bogoutil ; do
if [ ! -x $path/$prog ] ; then
   echo "$path/$prog is not an executable program"
   error="y"
fi
done

if [ "$error" = "y" ] ; then
    exit
fi

export BOGOFILTER="$path/bogofilter"
export BOGOLEXER="$path/bogolexer"
export BOGOUTIL="$path/bogoutil"
export BOGOFILTER_DIR="test.d"

export CFG

cat >$CFG <<EOF
algorithm=fisher
strict_check=no
block_on_subnets=yes
tag_header_lines=yes
terse_format=%1.1c %8.2d
spamicity_tags=1, 0, ?
spamicity_formats=%6.2e, %6.2e, %0.6f 
EOF

cnt=`ls $BOGOFILTER_DIR/goodlist.db $BOGOFILTER_DIR/spamlist.db 2>&1 | wc -l | tr -d " "`
if [ "$cnt" != "2" ] ; then
    echo Distributing messages and making dir $BOGOFILTER_DIR ...
    [ -d $BOGOFILTER_DIR ] || mkdir $BOGOFILTER_DIR
    mkdb 
    echo Done - distributing messages and making $BOGOFILTER_DIR.
fi

export target

function getco () {
    opts="-m$1 -o$2"
    shift ; shift ; shift
    res=`cat $* | $BOGOFILTER -t -c $CFG $opts -v 2>&1 | \
    perl -e ' $target = $ENV{"target"}; while (<>) { ' \
	 -e ' ($i, $d) = split; push @diffs, $d unless $i != 1; }' \
	 -e ' die "dainbramage" unless scalar @diffs > 15;' \
         -e ' @s = sort { $a <=> $b } (@diffs); $co = $s[$target];' \
	 -e ' while($co < 0.000001) { ++$target; $co = $s[$target]; }' \
	 -e ' printf("%8.6f %d",1.0-$s[$target],$target-1);'`
}

function wrapper () {
    v="-v"
    opts="-m$1 -o$2"
    shift ; shift ; shift
    res=`cat $1 | $BOGOFILTER -t -c $CFG $opts -v | grep -c $v '^1'`
}

function doit () {
    s=$1 ; shift
    md=$1 ; shift
    date=`date "+%m/%d %H:%M:%S"`
    echo -n $date "  "
    printf "%-6s %5.3f fpos..." $s $md
    getco $md 0.10 $s r0.ns.mc r1.ns.mc r2.ns.mc
    fpos=${res##* }; co=${res%% *}; let fpos=$fpos/3
    printf "%d at cutoff %8.6f, run0..." $fpos $co
    run=0; wrapper $md $co $s r0.sp.mc; fneg1=$res
    echo "$s $md $co $run $fpos $fneg1" >> $PARM_TBL
    printf "%3d  run1..." $fneg1
    run=1; wrapper $md $co $s r1.sp.mc; fneg2=$res
    echo "$s $md $co $run $fpos $fneg2" >> $PARM_TBL
    printf "%3d  run2..." $fneg2
    run=2; wrapper $md $co $s r2.sp.mc; fneg3=$res
    echo "$s $md $co $run $fpos $fneg3" >> $PARM_TBL
    printf "%3d"  $fneg3
    let fneg="$fneg1+$fneg2+$fneg3"
    printf " %4d\n"  $fneg
    printf "%s %s %-5s %5.3f fpos...%d at cutoff %8.6f, run0...%3d  run1...%3d  run2...%3d %4d\n" \
	$date $s $md  $fpos $co $fneg1 $fneg2 $fneg3 $fneg >> $RESULTS
}

./sizes | tee $RESULTS
./sizes | grep mc > $PARM_TBL

for s in $svals; do
  echo robs=$s >> $CFG
  echo "" | tee -a $RESULTS
  $BOGOFILTER -t -c $CFG -o$s -Q | grep 00 | tee -a $RESULTS
  for md in $mvals; do
      doit $s $md
  done
done

# get 10 best results (lowest false positive count)
(echo "" ; \
echo "Top 10 results" ; \
grep fpos < $RESULTS | sort +11n | head -10 ) | tee -a $RESULTS

date "+%m/%d %H:%M:%S"
